using System;

namespace CreditProcessLib
{
	/// <summary>
	/// Processes credit requests. If number if valid and the
	/// amount can be charged, charges and returns a charge code.
	/// </summary>
	public class CreditProcessor
	{
		private ICardNumberValidator cardValidator;
		private ICreditChargeEngine creditCharge;

		public void SetCreditChargeEngine(ICreditChargeEngine charge)
		{
			creditCharge = charge;
		}

		/// <summary>
		/// Creates a CreditCardProcessor.
		/// </summary>
		/// <param name="validator">The Card Number Validator to be used.</param>
		public CreditProcessor(ICardNumberValidator validator)
		{
			cardValidator = validator;
		}

		internal void validateCardNumber(string cardNumber)
		{
			if (!cardValidator.IsValid(cardNumber))
				throw new InvalidCreditCardNumberException();
		}

		/// <summary>
		/// Charges the credit card for the specified amount.
		/// Increases the card balance by the amount specified.
		/// </summary>
		/// <param name="cardNumber">Credit card number to be charged</param>
		/// <param name="amount">Charge amount</param>
		/// <returns>Charge transaction number or charge code</returns>
		/// <exception cref="CreditOverflowException">If charge plus current balance exceeds credit limit.</exception>"
		/// <exception cref="InvalidCreditCardNumberException">If given card number if not valid.</exception>
		public int Charge(string cardNumber, double amount)
		{
			validateCardNumber(cardNumber);

			if(!creditCharge.Charge(cardNumber, amount))
			{
				throw new CreditOverflowException();
			}

			return 1;
		}
	}
}
